// Container.java
//
// 2000-09-08 MK Added new Licensetext
// 2000-08-25 removeComponent fixed
// 2000-08-14 SH added several methods on request
// 2000-08-06 SH getComponentCount, and getInsets added
// 2000-05-20 SH isAncestorOf added
// 2000-05-01 SH foreground/background support completed
// 2000-03-06 SH add (Component, Object) fixed
// 2000-01-30 SH add (Component, Object) and isVisible () added
// 1999-11-26 SH added invalidate to add method 
//           (bug rep. by Morten Green Hermansen)
//	
//#include ..\..\license.txt
//
// kAWT version 0.96
//
// Copyright (C) 1999-2000 by Michael Kroll & Stefan Haustein GbR, Essen
//
// Contact: kawt@kawt.de
// General Information about kAWT is available at: http://www.kawt.de
//
// Using kAWT for private and educational and in GPLed open source
// projects is free. For other purposes, a commercial license must be
// obtained. There is absolutely no warranty for non-commercial use.
//
//
// 1. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO
//    WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
//    LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
//    HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT
//    WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT
//    NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
//    FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE
//    QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
//    PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
//    SERVICING, REPAIR OR CORRECTION.
//   
// 2. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
//    WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY
//    MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE
//    LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
//    INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
//    INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
//    DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU
//    OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY
//    OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN
//    ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//   
//    END OF TERMS AND CONDITIONS
// 
//#endinclude


package java.awt;

import java.util.Vector;
import java.awt.event.*;


public class Container extends Component {

  static Insets nullInsets = new Insets (0, 0, 0, 0);

    Vector children = new Vector ();
    //    Vector constraints;
    LayoutManager layoutManager = FlowLayout.defaultLayout;
    protected Insets insets = nullInsets;

    public Container() {
    }


    public Container (LayoutManager layoutManager) {
	this ();
	this.layoutManager = layoutManager;
    }


    public Component add (Component component) {
	add ((String) null, component);
	return component;
    }

    
    public Component add (String where, Component c) {
	children.addElement(c);
	c.parent = this;
	//	constraints.add (where);
	if (layoutManager != null) 
	    layoutManager.addLayoutComponent (where, c);
	invalidate ();
	return c;
    }


    public void add (Component c, Object where) {
	add ((String) where, c);
    }




  /*  included in validate directly in order to save stack space
  public void doLayout () {
    if (layoutManager != null)
	  layoutManager.layoutContainer (this);
  }
  */

  public Component findComponentAt (int px, int py) {
	
	if ((!visible) || !contains (px, py)) return null;

	for (int i = 0; i < children.size(); i++) {
	    Component c = (Component) children.elementAt (i);

	    c = c.findComponentAt (px - c.x, py - c.y);
	    
	    if (c != null) return c;
	}
	return this;
  }

    
    public Component getComponent (int i) {
	return (Component) children.elementAt (i);
    }

    public int getComponentCount () {
	return children.size ();
    }

    public Component [] getComponents () {
	Component [] result = new Component [children.size ()];
	for (int i = 0; i < result.length; i++) 
	    result [i] = getComponent (i);
	return result;
    }

    public Insets getInsets () {
	return insets;
    }

  
    public Dimension getMinimumSize () {
	return layoutManager != null 
	    ? layoutManager.minimumLayoutSize (this)
	    : getSize ();
    }


    public boolean isAncestorOf (Component c) {
	do {
	    if (c.parent == this) return true;
	    c = c.parent;
	}
	while (c != null);
	return false;
    }

  public LayoutManager getLayout () {
	return layoutManager;
  }


  public void paintAll (Graphics g) {
	if (visible && g != null) {
	    
	    if (!valid) doLayout ();

	    paint (g.create ());

	    //    paintComponents (g);
	    //  save some stack space (uahrg!)

	    for (int i = 0; i < children.size(); i++) {
		Component child = (Component) children.elementAt (i);
		if (child.visible && g.hitClip 
		    (child.x, child.y, child.w, child.h)) {
		    
		    Graphics g2 = g.create 
			(child.x, child.y, child.w, child.h);
		    //if (g2.
		    
		    if (!child.background.equals (background)) 
			child.update (g2);
		    else
			child.paintAll (g2);
		}
	    }
	}
  }
    
             
  public void paintComponents (Graphics g) {
	
	for (int i = 0; i < children.size(); i++) {
	    Component child = (Component) children.elementAt (i);
	    child.paintAll 
		(g.create (child.x, child.y, child.w, child.h));
	}
  }
    

    public void removeAll () {
	for (int i = children.size()-1; i >= 0; i--) 
	    remove ((Component) children.elementAt (i));
    }
 
    public void remove(Component c) {
	children.removeElement (c);
	if (layoutManager != null) 
	    layoutManager.removeLayoutComponent (c);
	invalidate ();
    }


    public void setLayout (LayoutManager mgr) {
	layoutManager = mgr;
	invalidate ();
	
	/*for (int i = 0; i < children.size (); i++) 
	  mgr.addLayoutComponent ((String) constraints.get (i), 
	  (Component) children.get (i));
	*/
    }

    public void doLayout () {

	if (layoutManager != null) layoutManager.layoutContainer (this);
	//doLayout ();
	for (int i = 0; i < children.size (); i++) 
	    ((Component) children.elementAt (i)).doLayout ();
	
	valid = true;
    }


    public void validate () {
	if (!valid) {
	    doLayout (); 
	    repaint ();
	}
    }


    public void update (Graphics g) {
	if (g== null) return;
	
	g.setColor (background);
	g.fillRect (0, 0, w, h);
	g.setColor (foreground);
	paintAll (g);
    }
}
